#######################################################################################################
# R code for fitting linear model changepoint regression to the data from each individual separately in 
# the Excel file "ALS.xlsx"
# The choice of treatment and sex to which the model is fitted requires changes in sections 5, 11 and 12
# Authors: 
#######################################################################################################
# 1) Required R packages
#######################################################################################################
  
  library(MASS)
  library(Matrix)
  library(lattice)
  library(latticeExtra)
  library(nlme)
  library(readxl)       # To read xlx files
  library(clubSandwich) # To obtain robust standard error [Liang & Zeger (1986)]
  
  #######################################################################################################
  # 2) Clean console and plots
  #######################################################################################################
  
  dev.off()
  rm(list = ls())
  cat("\014")
  options("scipen" = 1000, digits = 18, "width" = 200, encoding = "utf-8")
  
  #######################################################################################################
  # 3) Use setwd and specify the path to the data set and read data set
  #######################################################################################################
  
  setwd("~/Dropbox/Changepoint/Artigo/JASA/SuplementaryMaterial")
  
  alsdata <- read_excel("ALS.xlsx",'RotaRod', na = "NA")
  
  #######################################################################################################
  
  # 4) Transform data from wide format to long format
  #######################################################################################################
  
  vecalsdata <- reshape(alsdata, direction = "long", varying = 4:18, v.names = "Time", timevar = "Week", idvar = "ind", times = 8:22)
  vecalsdata$id <- vecalsdata$subject
  vecalsdata <- vecalsdata[order(vecalsdata$id),]
  
  #######################################################################################################
  
  # 5) Select treatment and sex to analyse
  
  #######################################################################################################
  
  #vecalsdata <- subset(vecalsdata, vecalsdata$treatment == 'HBSS' & vecalsdata$sex == 'F')
  # vecalsdata <- subset(vecalsdata, vecalsdata$treatment == 'HBSS' & vecalsdata$sex == 'M')
  # vecalsdata <- subset(vecalsdata, vecalsdata$treatment == 'MSC' & vecalsdata$sex == 'M')
  # vecalsdata <- subset(vecalsdata, vecalsdata$treatment == 'MSC' & vecalsdata$sex == 'F')
  # vecalsdata <- subset(vecalsdata, vecalsdata$treatment == 'Pericyte' & vecalsdata$sex == 'M')
  vecalsdata <- subset(vecalsdata, vecalsdata$treatment == 'Pericyte' & vecalsdata$sex == 'F')
  vecalsdata <- na.exclude(vecalsdata)
  L1 <- min(vecalsdata$Week)
  L2 <- max(vecalsdata$Week)
  
  #######################################################################################################
  
  # 6) Construction of profile plot
  #######################################################################################################
  
  xyplot(Time ~ Week, groups = subject, pch = 16, 
         par.settings = standard.theme(quartz,color = F), 
         scales = list(x = list(relation = 'same'), 
                       y = list(relation = 'same')), 
         type = 'l', lty = 1, 
         col = 'black', as.table = TRUE, 
         panel = function(x, y, col, ...){ 
           panel.xyplot(x, y, col = "black", ...) 
           panel.loess(x, y, span = .65, degree = 1, family = "symmetric", lwd = 4)}, 
         ylab = "Time (s)", xlab = "Weeks", sub = 'Loess in bold', 
         na.action = na.omit, data = vecalsdata, main = 'Rotarod')
  
  #######################################################################################################
  # 7) Construction of mean profile plot (optional)
  #######################################################################################################
  
  perf.mean <- aggregate(vecalsdata$Time, by = list(vecalsdata$Week), FUN = mean, na.rm = TRUE)
  colnames(perf.mean) <- c("Week","Time")
  perf.mean <- subset(perf.mean, perf.mean$Time > 0)
  plot(Time ~ Week, data = perf.mean, type = 'l', ylim = c(0,200))
  
  #######################################################################################################
  
  # 8) Fitting random changepoint regression model to selected data
  # Use setwd and specify the path to the model fitting function 
  #######################################################################################################
  
  setwd("~/Dropbox/Changepoint/Artigo/JASA/SuplementaryMaterial")
  source("seg2changepoint.R", encoding = "UTF-8")
  
  estimate.ind <- matrix(0,length(unique(vecalsdata$ind)),25)
  nind <- unique(vecalsdata$ind)
  covar.b <- matrix(0,3*length(unique(vecalsdata$ind)),3)
  for (k in 1:length(nind)) {
    data.set <- subset(vecalsdata,vecalsdata$ind == nind[k])
    var.indep <- data.set$Week
    aux1 <- min(var.indep)
    aux2 <- max(var.indep)
    p1_0 <- seq((aux1),(aux2),by = 0.25)
    p2 <- max(data.set$Week) # 20.5  max(data.set$Week)
    sel.mod <- matrix(0,length(p1_0),2)
    for (i in 1:length(p1_0)) {
      result <- try(quad_change_point.LM(data.set,data.set$Week,p1_0[i],p2),TRUE)
      if (inherits(result,"try-error") ) {
        sel.mod[i,] <- c(psi1 = p1_0[i],mspe = -1)
      }else if (result$iter >= 50000){
        sel.mod[i,] <- c(psi1 = p1_0[i],mspe = -1)
      }else{
        aux <- result$mspe
        sel.mod[i,] <- c(psi1 = p1_0[i],mspe = aux)
      }
    }
  
    val <- subset(sel.mod,sel.mod[,2] > -1)
    result.f <- try(quad_change_point.LM(data.set,data.set$Week,
                                  subset(val,val[,2] == min(val[,2]))[1,1],
                                  p2),TRUE)
    aux.ind <- c(result.f$estimates,result.f$psi.ind,result.f$Std.Error,
                          result.f$fitted.ind)
    covar.b[(3*k - 2):(3*k),1:3] <- result.f$vcov.beta 
    if (length(aux.ind) <= dim(estimate.ind)[2]) {
        for (t in 1:length(aux.ind)) {
          estimate.ind[k,t] <- aux.ind[t]
        }   
    }
  }
  #==============================================================================#
  #==============================================================================#
  #
################################################################################
################################################################################
# Average of the individual parameters
apply(estimate.ind,2,sd)[1:5]/sqrt(length(nind))
#==============================================================================#
#==============================================================================#
# Plotting individual curves with predicted values
  
valores.pred <- function (x,par){
  a <- par[1]
  g <- par[2]
  psif1 <- par[3]
  psif2 <- par[4]
  alp <- ifelse(x < psif2,1,0)
  U <- (ifelse(x >= psif1 & x < psif2,x - psif1, 0))^2
  a*alp + g*U
}

p1 <- floor(length(nind)/2) - 1
p2 <- floor(length(nind)/2) - 2

par(mfrow = c(p1,p2))

for (v in 1:length(nind)) {
  dat <- subset(vecalsdata,vecalsdata$ind == nind[v])
  parametros <- estimate.ind[v,c(1,2,4,5)]
  plot(dat$Week,dat$Time,ylim = c(0,200),xlim = c(7,25),xlab = 'Time',
  ylab = 'Week',main = paste(nind[v]))
  xp <- seq(8,25,0.001)
  ypred <- subset(valores.pred(xp,parametros),
  valores.pred(xp,parametros) != 0)
  lines(xp[1:length(ypred)],ypred)
  
}

setwd("~/Dropbox/Changepoint/Figuras")
dev.copy(pdf,'Ind_MSC_M.pdf')
dev.off()

#==============================================================================#
# Saving individual estimates in an Excel file
library("WriteXLS")


colnames(estimate.ind) <- c('alpha','gama','lambda','psi1','psi2',
's.e.alpha','s.e.gama','s.e.lambda','t8	t9','t10','t11','t12','t13',
't14','t15','t16','t17','t18','t19','t20','t21','t22')


setwd("~/Dropbox/Changepoint/Dados")
library("WriteXLS")
 WriteXLS(as.data.frame(cbind(nind,estimate.ind)), "Estimate_Ind_MSC_M.xlsx",
          SheetNames = "MSC_M",AdjWidth = TRUE, BoldHeaderRow = TRUE,
          row.names = FALSE)




#==============================================================================#

#==============================================================================#

library("WriteXLS")
 WriteXLS(as.data.frame(covar.b), "Estimate_Ind_cov_par_MSC_M.xlsx",
          SheetNames = "MSC_M",AdjWidth = TRUE, BoldHeaderRow = TRUE,
          row.names = FALSE)

#==============================================================================#


