#######################################################################################################
# R code for fitting random changepoint regression model to data available in the Excel file
# "ALS.xlsx"
# The choice of treatment and sex to which the model is fitted requires changes in sections 5, 11 and 12
# Authors: 
#######################################################################################################
# 1) Required R packages
#######################################################################################################

library(MASS)
library(Matrix)
library(lattice)
library(latticeExtra)
library(nlme)
library(readxl)       # To read xlx files
library(clubSandwich) # To obtain robust standard error [Liang & Zeger (1986)]

#######################################################################################################
# 2) Clean console and plots
#######################################################################################################

dev.off()
rm(list = ls())
cat("\014")
options("scipen" = 1000, digits = 18, "width" = 200, encoding = "utf-8")

#######################################################################################################
# 3) Use setwd and specify the path to the data set and read data set
#######################################################################################################

setwd("~/Dropbox/Changepoint/Data")

alsdata <- read_excel("ALS.xlsx",'RotaRod', na = "NA")

#######################################################################################################
# 4) Transform data from wide format to long format
#######################################################################################################

vecalsdata <- reshape(alsdata, direction = "long", varying = 4:18, v.names = "Time", timevar = "Week", 
                      idvar = "ind", times = 8:22)
vecalsdata$id <- vecalsdata$subject
vecalsdata <- vecalsdata[order(vecalsdata$id),]

#######################################################################################################
# 5) Select treatment and sex to analyse
#######################################################################################################

#vecalsdata <- subset(vecalsdata, vecalsdata$treatment == 'HBSS' & vecalsdata$sex == 'F')
# vecalsdata <- subset(vecalsdata, vecalsdata$treatment == 'HBSS' & vecalsdata$sex == 'M')
# vecalsdata <- subset(vecalsdata, vecalsdata$treatment == 'MSC' & vecalsdata$sex == 'M')
# vecalsdata <- subset(vecalsdata, vecalsdata$treatment == 'MSC' & vecalsdata$sex == 'F')
# vecalsdata <- subset(vecalsdata, vecalsdata$treatment == 'Pericyte' & vecalsdata$sex == 'M')
vecalsdata <- subset(vecalsdata, vecalsdata$treatment == 'Pericyte' & vecalsdata$sex == 'F')
vecalsdata <- na.exclude(vecalsdata)
L1 <- min(vecalsdata$Week)
L2 <- max(vecalsdata$Week)

#######################################################################################################
# 6) Construction of profile plot
#######################################################################################################

xyplot(Time ~ Week, groups = subject, pch = 16, 
       par.settings = standard.theme(quartz,color = F), 
       scales = list(x = list(relation = 'same'), 
                     y = list(relation = 'same')), 
                     type = 'l', lty = 1, 
                     col = 'black', as.table = TRUE, 
                     panel = function(x, y, col, ...){ 
                     panel.xyplot(x, y, col = "black", ...) 
                     panel.loess(x, y, span = .65, degree = 1, family = "symmetric", lwd = 4)}, 
                     ylab = "Time (s)", xlab = "Weeks", sub = 'Loess in bold', 
                     na.action = na.omit, data = vecalsdata, main = 'Rotarod')

#######################################################################################################
# 7) Construction of mean profile plot (optional)
#######################################################################################################

perf.mean <- aggregate(vecalsdata$Time, by = list(vecalsdata$Week), FUN = mean, na.rm = TRUE)
colnames(perf.mean) <- c("Week","Time")
perf.mean <- subset(perf.mean, perf.mean$Time > 0)
plot(Time ~ Week, data = perf.mean, type = 'l', ylim = c(0,200))

#######################################################################################################
# 8) Fitting random changepoint regression model to selected data
# Use setwd and specify the path to the model fitting function 
#######################################################################################################

setwd("~/Dropbox/Changepoint/Software/Algorithmfunction")
source("seg2changepoint.R", encoding = "UTF-8")

p1_0 <- seq(L1,L2,by = 0.25)
p2 <- L2
sel.mod <- matrix(0,length(p1_0),2)

# result <- try(quad_change_point(vecdados,vecdados$Week,12.5,p2),TRUE)
for (i in 1:length(p1_0)) {
  result <- try(quad_change_point(vecalsdata, vecalsdata$Week, p1_0[i],p2), TRUE)
  if (inherits(result,"try-error")) {
    sel.mod[i,] <- c(psi1 = p1_0[i], mspe = -1)
  }else if (result$iter >= 500) {
    sel.mod[i,] <- c(psi1 = p1_0[i], mspe = -1)
  }else{
    aux <- result$mspe
    sel.mod[i,] <- c(psi1 = p1_0[i], mspe = aux)
  }
}
val <- subset(sel.mod,sel.mod[,2] > -1)
result <- try(quad_change_point(vecalsdata, vecalsdata$Week,
                                subset(val, val[,2] == min(val[,2]))[1,1], p2), TRUE)

#######################################################################################################
# 9) Diagnostic plots from Singer et al. (2017)
# Use setwd and specify the path to the function that generates the 
# diagnostic plots for gaussian linear mixed models fitted via lme function 
#######################################################################################################

setwd("~/Dropbox/Changepoint/Software/Algorithmfunction")

source("residdiag_nlme_v39_BETA.R", encoding = "UTF-8")
resid <- residdiag.nlme(result$fm, limit = 2, plotid = 1:16)

#######################################################################################################
# 10) Plots with predicted curves
#######################################################################################################

summary(result$fm)$tTable
result$psi.ti

getVarCov(result$fm,'random.effects')
m0 <- result$fm
L1new <- 8     # min(vecalsdata$Week)
L2new <- 25    # max(vecalsdata$Week)

psi1 <- result$psi.ti[,1]
psi2 <- result$psi.ti[,2]

psi1i <- result$psi.ind[,1]
psi2i <- result$psi.ind[,2]

pred <- with(vecalsdata,
             expand.grid(xij = seq(L1new, L2new, by = 0.1),
                         subject = (unique(id))))
pred$id <- pred$subject
aux <- as.numeric(table(pred$subject))[1]
cutpoint1 <- rep(psi1i, each = aux)
cutpoint2 <- rep(psi2i, each = aux)
pred$V <- rep(0,dim(pred)[1]) 

pred$U <- ifelse(pred$xij >= cutpoint1 & pred$xij < cutpoint2, pred$xij - cutpoint1, 0)
pred$U <- pred$U^2
pred$alpha <- ifelse(pred$xij < cutpoint2,1,0)
pred$Time1 <- predict(m0, newdata = pred, level = 1)
pred$alpha <- ifelse(pred$xij < psi2,1,0)
pred$U <- ifelse(pred$xij >= psi1 & pred$xij < psi2, pred$xij - psi1, 0)
pred$U <- pred$U^2
pred$Time0 <- predict(m0, newdata = pred, level = 0)

pred0 <- pred[,c(1,2,3,8)]
pred1 <- pred[,c(1,2,3,7)]

pred0$subjectf <- as.factor(pred0$subject)
pred1$subjectf <- as.factor(pred1$subject)

pred0 <- subset(pred0,pred0$Time0 > 0)
pred1 <- subset(pred1,pred1$Time1 > 0)

#######################################################################################################
# 11) Average of the cutpoints obtained from fitting separate individual curves
#######################################################################################################

# psi1.med.ind <-  13.360296538816712086   # Valid for HBSS female  
# psi2.med.ind <-  21.542996613646021586   # Valid for HBSS female

# psi1.med.ind <- 14.0892591069096  # Valid for HBSS male 
# psi2.med.ind <-  16.9068459682543 # Valid for HBSS male
# psi1.med.ind <- 13.5551648502882  # Valid for MSC male 
# psi2.med.ind <- 16.1871674687416  # Valid for MSC male
# psi1.med.ind <- 14.8520099890141  # Valid for MSC female 
# psi2.med.ind <- 18.8675891610063  # Valid for MSC female
# psi1.med.ind <- 14.1652821475004  # Valid for Pericyte male 
# psi2.med.ind <- 19.5523934472056  # Valid for Pericyte male
 psi1.med.ind <- 14.5422701358034  # Valid for Pericyte female 
 psi2.med.ind <- 18.0262258491462  # Valid for Pericyte female

#######################################################################################################
# 12) Computation of the predicted values considering the average individual estimates
#######################################################################################################

alpha.ind <- ifelse(pred$xij < psi2.med.ind,1,0)
U.ind <- ifelse(pred$xij >= psi1.med.ind & pred$xij < psi2.med.ind,
                pred$xij - psi1.med.ind, 0)
U.ind <- U.ind^2
# pred$Time.ind <- (154.949569869494553132*alpha.ind) - (15.172662394054000856*U.ind)   # Valid for HBSS female
# pred$Time.ind <- (165.978145546973*alpha.ind) - (30.1952996498993*U.ind) # Valid for HBSS male
# pred$Time.ind <- (170.033440797582*alpha.ind) - (61.1499115094186*U.ind) # Valid for MSC male
# pred$Time.ind <- (168.447289717559*alpha.ind) - (26.9510047066699*U.ind) # Valid for MSC female
# pred$Time.ind <- (164.186076639466*alpha.ind) - (45.4962627582939*U.ind) # Valid for Pericyte male
 pred$Time.ind <- (171.546673288862*alpha.ind) - (30.5993458344956*U.ind) # Valid for Pericyte female
# 	-

pred2 <- pred[, c(1,2,3,9)]
pred2 <- subset(pred2, pred2$Time.ind > 0)

#######################################################################################################
# 13) Construction of the overlayed pedicted individual, typical individual and averaged individual plots
#######################################################################################################

xyplot(Time ~ Week | factor(id), data = vecalsdata, strip = T, cex = 0.5,
       ylab = 'Time (sec)',
       scales = list(y = list(at = seq(0,200,25), limits = c(-10,200)),
                     x = list(at = seq(10,25,5),  limits = c(5,25))),
                     key = list(columns = 2,
                     text = list(c("Subject specific","Typical subject",
                                "Average based on individual models")),
                     lines = list(type = "l", col = c("black","black","black"),
                               lty = c(2,1,6))),
                     par.settings = standard.theme(quartz, color = F), as.table = TRUE) +
  as.layer(xyplot(Time0 ~ xij|subjectf, data = pred0, type = "l",lty = 1, 
                  col = 1, order = T)) +
  as.layer(xyplot(Time1 ~ xij|subjectf, data = pred1, type = "l",lty = 2, 
                  col = 1, order = T)) + 
  as.layer(xyplot(Time.ind ~ xij|subject, data = pred2, type = "l",lty = 6, 
                  col = 1, order = T))

#######################################################################################################
# 14) Plot of fitted individual, typical individual and marginal profiles (optional)
#######################################################################################################

xyplot(Time ~ Week , data = vecalsdata, strip = T,cex = 0.5,
       ylim = c(-20,240),
       key = list(columns = 2,text = list(c("TI", "SS")),
             lines = list(type = "l", col = c("black","black"), lty = 1:2)),
             par.settings = standard.theme(quartz, color = F), as.table = TRUE) +
             as.layer(xyplot(Time0 ~ xij|subjectf, data = pred0, type = "l", lty = 1, 
                      col = 1, order = T, lwd = 3, par.settings = standard.theme(quartz, color = F), 
                      as.table = TRUE)) +
                      as.layer(xyplot(Time1 ~ xij, groups = id, data = pred1, type = "l",ltya = 2,
                      strip = T,col = 1, order = TRUE,
                      par.settings = standard.theme(quartz, color = F), 
                      as.table = TRUE),
                      x.same = TRUE, y.same = TRUE,under = F )

####################################################################################################### 
# 15) Std. errors for psi_1 and psi_2 computed via Delta method
#######################################################################################################

beta.cov <- result$fm$varFix
beta <- (result$fm$coefficients$fixed)
diff.psi1.lambda <- ((L2 - L1)*exp(beta[3]))/((1 + exp(beta[3]))^2)
diff.psi2.alpha <- (1/(2*beta[1]))*sqrt(-beta[1]/beta[2])
diff.psi2.gama <- (1/(2*beta[1]))*((-beta[1]/beta[2])^(3/2))
diff.psi2.lambda <- diff.psi1.lambda

l1 <- c(1,0,0)
l2 <- c(0,1,0)
l3 <- c(0,0,1)
l4 <- c(0,0,diff.psi1.lambda)
l5 <- c(diff.psi2.alpha,diff.psi2.gama,diff.psi2.lambda)
m.G <- matrix(c(l1,l2,l3,l4,l5),5,3,byrow = TRUE)
G.varBeta.Gt <- m.G %*% beta.cov %*% t(m.G)
sqrt(diag(G.varBeta.Gt))

#######################################################################################################
# 16) Robust Std. errors for psi_1 and psi_2 computed via Delta method
#######################################################################################################

vLZ <- vcovCR(result$fm, type = "CR0")
vLZ
sqrt(diag(vLZ))

beta.cov.LZ <- vLZ
beta <- (result$fm$coefficients$fixed)
diff.psi1.lambda <- ((L2 - L1)*exp(beta[3]))/((1 + exp(beta[3]))^2)
diff.psi2.alpha <- (1/(2*beta[1]))*sqrt(-beta[1]/beta[2])
diff.psi2.gama <- (1/(2*beta[1]))*((-beta[1]/beta[2])^(3/2))
diff.psi2.lambda <- diff.psi1.lambda

l1 <- c(1,0,0)
l2 <- c(0,1,0)
l3 <- c(0,0,1)
l4 <- c(0,0,diff.psi1.lambda)
l5 <- c(diff.psi2.alpha,diff.psi2.gama,diff.psi2.lambda)
m.G <- matrix(c(l1,l2,l3,l4,l5),5,3,byrow = TRUE)
G.varBeta.Gt.LZ <- m.G %*% beta.cov.LZ %*% t(m.G)
sqrt(diag(G.varBeta.Gt.LZ))

##################################################################################################
##################################################################################################

